/**
  ******************************************************************************
  * File Name          : main.c
  * Description        : Main program body
  ******************************************************************************
  *
  * COPYRIGHT(c) 2017 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "stm32l0xx_hal.h"

/* USER CODE BEGIN Includes */

/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/
ADC_HandleTypeDef hadc;

LCD_HandleTypeDef hlcd;

LPTIM_HandleTypeDef hlptim1;

TIM_HandleTypeDef htim6;
TIM_HandleTypeDef htim21;

UART_HandleTypeDef huart1;

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/
#define LPT_PERIOD 8700 //=30 sekund - z takim opoznieniem wykonywane sa kolejne pomiary
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
void Error_Handler(void);
static void MX_GPIO_Init(void);
static void MX_LCD_Init(void);
static void MX_LPTIM1_Init(void);
static void MX_USART1_UART_Init(void);
static void MX_TIM6_Init(void);
static void MX_TIM21_Init(void);
static void MX_ADC_Init(void);

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/
void ZmTryb(void);
void Tryb0(void);
void Tryb1(void);
void Tryb2(void);
void Tryb3(void);
void Tryb4(void);
void Tryb5(void);
void BtnExti(void);
void BtnGpio(void);
void BtnWakeUp(void);
void StopMode(void);
void StandBy(void);
void PiezoStart(uint16_t);
void BoostOn(void);
void BoostOff(void);
void WaitUs(uint16_t);
uint16_t OneWireRST(void);
void OneWireSendBit(uint16_t);
uint16_t OneWireReadBit(void);
void OneWireSend(uint16_t);
uint16_t OneWireRead(void);
void DispLCD(int16_t);
uint32_t Lookup2(uint16_t cyfra);
uint32_t Lookup3(uint16_t cyfra);
void RelOff(void);
void RelOn(void);
void Keys(void);
void Btn1(void);
void Btn2(void);
void Btn3(void);
void Btn4(void);
void Measure(void);
void ConvertTemp(void);
void ReadTemp(void);
void CtrlTemp(void);
void RstTimeout(void);
void Alarm(void);
void CtrlBat(void);
/* USER CODE END PFP */

/* USER CODE BEGIN 0 */
uint16_t tryb = 0;
uint16_t tryb_next = 0;
uint16_t timeout;
volatile uint16_t temp_odsw;
volatile uint16_t relDelay = 0;
volatile uint16_t measDelay = 1;
volatile uint16_t blinkDelay = 0;
uint16_t blink = 0;
uint16_t key1 = 0;
uint16_t key2 = 0;
uint16_t key3 = 0;
uint16_t key4 = 0;
uint16_t relState = 1;
uint16_t alarm = 0;
uint16_t repeat = 0;
volatile uint16_t repeat2 = 0;
uint16_t repeat3 = 0;
uint16_t ds_error = 0;
uint16_t ds_85 = 0;
int16_t temperature = (-100);
volatile int16_t prog_temp;
uint16_t piezo_cnt = 0;
volatile uint16_t piezoDelay = 0;
uint16_t rel_turnoff = 0;
uint16_t led_meas;
uint16_t bat_meas = 0;
uint16_t calibrated = 0;

/* USER CODE END 0 */

int main(void)
{

  /* USER CODE BEGIN 1 */
  /* USER CODE END 1 */

  /* MCU Configuration----------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* Configure the system clock */
  SystemClock_Config();

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_LCD_Init();
  MX_LPTIM1_Init();
  MX_USART1_UART_Init();
  MX_TIM6_Init();
  MX_TIM21_Init();
  MX_ADC_Init();

  /* USER CODE BEGIN 2 */
	Tryb0();
	DispLCD(200);
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
  /* USER CODE END WHILE */

  /* USER CODE BEGIN 3 */
		Keys(); 
		ZmTryb();
		Measure();
		CtrlTemp();
		CtrlBat();
	}
		
  /* USER CODE END 3 */

}

/** System Clock Configuration
*/
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_PeriphCLKInitTypeDef PeriphClkInit;

  __HAL_RCC_PWR_CLK_ENABLE();

  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI|RCC_OSCILLATORTYPE_LSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = 16;
  RCC_OscInitStruct.LSIState = RCC_LSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV2;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;
  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    Error_Handler();
  }

  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART1|RCC_PERIPHCLK_RTC
                              |RCC_PERIPHCLK_LPTIM1;
  PeriphClkInit.Usart1ClockSelection = RCC_USART1CLKSOURCE_PCLK2;
  PeriphClkInit.RTCClockSelection = RCC_RTCCLKSOURCE_LSI;
  PeriphClkInit.LptimClockSelection = RCC_LPTIM1CLKSOURCE_LSI;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }

  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/* ADC init function */
static void MX_ADC_Init(void)
{

  ADC_ChannelConfTypeDef sConfig;

    /**Configure the global features of the ADC (Clock, Resolution, Data Alignment and number of conversion) 
    */
  hadc.Instance = ADC1;
  hadc.Init.OversamplingMode = DISABLE;
  hadc.Init.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV2;
  hadc.Init.Resolution = ADC_RESOLUTION_12B;
  hadc.Init.SamplingTime = ADC_SAMPLETIME_1CYCLE_5;
  hadc.Init.ScanConvMode = ADC_SCAN_DIRECTION_FORWARD;
  hadc.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc.Init.ContinuousConvMode = DISABLE;
  hadc.Init.DiscontinuousConvMode = DISABLE;
  hadc.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
  hadc.Init.DMAContinuousRequests = DISABLE;
  hadc.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
  hadc.Init.Overrun = ADC_OVR_DATA_OVERWRITTEN;
  hadc.Init.LowPowerAutoWait = ENABLE;
  hadc.Init.LowPowerFrequencyMode = DISABLE;
  hadc.Init.LowPowerAutoPowerOff = ENABLE;
  if (HAL_ADC_Init(&hadc) != HAL_OK)
  {
    Error_Handler();
  }

    /**Configure for the selected ADC regular channel to be converted. 
    */
  sConfig.Channel = ADC_CHANNEL_VREFINT;
  sConfig.Rank = ADC_RANK_CHANNEL_NUMBER;
  if (HAL_ADC_ConfigChannel(&hadc, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }

}

/* LCD init function */
static void MX_LCD_Init(void)
{

  hlcd.Instance = LCD;
  hlcd.Init.Prescaler = LCD_PRESCALER_32;
  hlcd.Init.Divider = LCD_DIVIDER_17;
  hlcd.Init.Duty = LCD_DUTY_STATIC;
  hlcd.Init.Bias = LCD_BIAS_1_2;
  hlcd.Init.VoltageSource = LCD_VOLTAGESOURCE_INTERNAL;
  hlcd.Init.Contrast = LCD_CONTRASTLEVEL_0;
  hlcd.Init.DeadTime = LCD_DEADTIME_0;
  hlcd.Init.PulseOnDuration = LCD_PULSEONDURATION_0;
  hlcd.Init.HighDrive = LCD_HIGHDRIVE_0;
  hlcd.Init.BlinkMode = LCD_BLINKMODE_OFF;
  hlcd.Init.BlinkFrequency = LCD_BLINKFREQUENCY_DIV16;
  hlcd.Init.MuxSegment = LCD_MUXSEGMENT_DISABLE;
  if (HAL_LCD_Init(&hlcd) != HAL_OK)
  {
    Error_Handler();
  }

}

/* LPTIM1 init function */
static void MX_LPTIM1_Init(void)
{

  hlptim1.Instance = LPTIM1;
  hlptim1.Init.Clock.Source = LPTIM_CLOCKSOURCE_APBCLOCK_LPOSC;
  hlptim1.Init.Clock.Prescaler = LPTIM_PRESCALER_DIV128;
  hlptim1.Init.Trigger.Source = LPTIM_TRIGSOURCE_SOFTWARE;
  hlptim1.Init.OutputPolarity = LPTIM_OUTPUTPOLARITY_HIGH;
  hlptim1.Init.UpdateMode = LPTIM_UPDATE_IMMEDIATE;
  hlptim1.Init.CounterSource = LPTIM_COUNTERSOURCE_INTERNAL;
  if (HAL_LPTIM_Init(&hlptim1) != HAL_OK)
  {
    Error_Handler();
  }

}

/* TIM6 init function */
static void MX_TIM6_Init(void)
{

  TIM_MasterConfigTypeDef sMasterConfig;

  htim6.Instance = TIM6;
  htim6.Init.Prescaler = 8000;
  htim6.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim6.Init.Period = 15;
  if (HAL_TIM_Base_Init(&htim6) != HAL_OK)
  {
    Error_Handler();
  }

  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim6, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }

}

/* TIM21 init function */
static void MX_TIM21_Init(void)
{

  TIM_ClockConfigTypeDef sClockSourceConfig;
  TIM_MasterConfigTypeDef sMasterConfig;

  htim21.Instance = TIM21;
  htim21.Init.Prescaler = 8;
  htim21.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim21.Init.Period = 65535;
  htim21.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  if (HAL_TIM_Base_Init(&htim21) != HAL_OK)
  {
    Error_Handler();
  }

  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  if (HAL_TIM_ConfigClockSource(&htim21, &sClockSourceConfig) != HAL_OK)
  {
    Error_Handler();
  }

  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim21, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }

}

/* USART1 init function */
static void MX_USART1_UART_Init(void)
{

  huart1.Instance = USART1;
  huart1.Init.BaudRate = 115200;
  huart1.Init.WordLength = UART_WORDLENGTH_8B;
  huart1.Init.StopBits = UART_STOPBITS_1;
  huart1.Init.Parity = UART_PARITY_NONE;
  huart1.Init.Mode = UART_MODE_TX_RX;
  huart1.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart1.Init.OverSampling = UART_OVERSAMPLING_16;
  huart1.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart1.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart1) != HAL_OK)
  {
    Error_Handler();
  }

}

/** Configure pins as 
        * Analog 
        * Input 
        * Output
        * EVENT_OUT
        * EXTI
        * Free pins are configured automatically as Analog (this feature is enabled through 
        * the Code Generation settings)
*/
static void MX_GPIO_Init(void)
{

  GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOH_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOC, PIEZO_Pin|BOOST_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, REL_ON_Pin|REL_OFF_Pin|LCD_BL_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB, LED_MEAS_Pin|LED_ALARM_Pin|SENSOR_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pins : PIEZO_Pin BOOST_Pin */
  GPIO_InitStruct.Pin = PIEZO_Pin|BOOST_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pin : AC_ON_Pin */
  GPIO_InitStruct.Pin = AC_ON_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(AC_ON_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : PH0 PH1 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOH, &GPIO_InitStruct);

  /*Configure GPIO pins : BTN4_WKUP_Pin BTN3_Pin BTN1_Pin */
  GPIO_InitStruct.Pin = BTN4_WKUP_Pin|BTN3_Pin|BTN1_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : BTN2_Pin */
  GPIO_InitStruct.Pin = BTN2_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(BTN2_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : REL_ON_Pin REL_OFF_Pin LCD_BL_Pin */
  GPIO_InitStruct.Pin = REL_ON_Pin|REL_OFF_Pin|LCD_BL_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : LED_MEAS_Pin LED_ALARM_Pin */
  GPIO_InitStruct.Pin = LED_MEAS_Pin|LED_ALARM_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pin : SENSOR_Pin */
  GPIO_InitStruct.Pin = SENSOR_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_OD;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(SENSOR_GPIO_Port, &GPIO_InitStruct);

  /* EXTI interrupt init*/
  HAL_NVIC_SetPriority(EXTI0_1_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI0_1_IRQn);

  HAL_NVIC_SetPriority(EXTI2_3_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI2_3_IRQn);

  HAL_NVIC_SetPriority(EXTI4_15_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI4_15_IRQn);

}

/* USER CODE BEGIN 4 */
void ZmTryb(void) {
	int16_t prog_temp_eeprom;
	if (tryb_next != tryb) {
		if (tryb == 3) { //Przy wyjsciu z trybu ustawiania zapisz nowy prog alarmu w EEPROM
			prog_temp_eeprom = *(uint16_t *)(0x08080000);
			if (prog_temp_eeprom != prog_temp) {
				HAL_FLASHEx_DATAEEPROM_Unlock();
				HAL_FLASHEx_DATAEEPROM_Program(FLASH_TYPEPROGRAMDATA_HALFWORD,0x08080000,prog_temp);
				HAL_FLASHEx_DATAEEPROM_Lock();
			}
		}
		if (tryb_next == 1) {
			Tryb1();
		} else if (tryb_next == 2) {
			Tryb2();
		} else if (tryb_next == 3) {
			Tryb3();
		} else if (tryb_next == 4) {
			Tryb4();
		} else if (tryb_next == 5) {
			Tryb5();
		} else if (tryb_next == 6) {
			RelOff();
			StopMode();
		} else if (tryb_next == 7) {
			if (bat_meas < 3) {
				StopMode();
			} else {
				tryb_next = 5;
				alarm = 6;
			}
		} else if (tryb_next == 8) {
			StandBy();
		}
	}
}

void Tryb0(void) {
	uint32_t pm, pm2;
	char p;
	int16_t prog_temp_eeprom;
	__HAL_PWR_CLEAR_FLAG(PWR_FLAG_SB);
	__HAL_PWR_CLEAR_FLAG(PWR_FLAG_WU);
	HAL_PWR_DisableWakeUpPin(PWR_WAKEUP_PIN1);
	RCC->CFGR |= RCC_CFGR_STOPWUCK;
	BtnGpio();
	LCD->CR |= LCD_CR_LCDEN;
	TIM6->SR = 0;
	TIM6->CNT = 0;
	PWR->CR &= ~PWR_CR_FWU;
	key1 = 100;
	key2 = 100;
	key3 = 100;
	key4 = 100;
	temp_odsw |= 0x0002;
	HAL_ADCEx_EnableVREFINT();
	HAL_ADCEx_Calibration_Start(&hadc, ADC_SINGLE_ENDED);	
	calibrated = 1;
	TIM6->DIER = TIM6->DIER | TIM_DIER_UIE;
	TIM6->CR1 = TIM6->CR1 | TIM_CR1_CEN;
	TIM21->CR1 = TIM21->CR1 | TIM_CR1_CEN;
	HAL_LPTIM_Counter_Start_IT(&hlptim1, LPT_PERIOD); //Wlacz LPTIM
	BoostOn();
	RelOff();
	alarm = 0;
	repeat = 0;
	PiezoStart(1);
	GPIOA->BSRR = GPIO_PIN_15;
	GPIOB->BSRR = GPIO_PIN_6;
	GPIOB->BSRR = GPIO_PIN_7;
	LCD->FCR &= ~0x00030000;
	//wczytaj prog alarmu z pamieci EEPROM
	prog_temp_eeprom = *(uint16_t *)(0x08080000);
	if ((prog_temp_eeprom > 120) | (prog_temp_eeprom < (-50))) { //Jesli wartosc jest nieprawidlowa, zapisz domyslnie 70 stopni
		prog_temp = 70;
		HAL_FLASHEx_DATAEEPROM_Unlock();
		HAL_FLASHEx_DATAEEPROM_Program(FLASH_TYPEPROGRAMDATA_HALFWORD,0x08080000,prog_temp);
		HAL_FLASHEx_DATAEEPROM_Lock();
	} else {
		prog_temp = prog_temp_eeprom;
	}
	RstTimeout();
	//USART1->CR1 |= USART_CR1_UE;
}

void Tryb1(void) {
	tryb = 1;
	LCD->FCR &= ~0x00030000;
	temp_odsw |= 0x0002;
	RelOff();
	repeat = 0;
	repeat3 = 0;
	GPIOA->BSRR = GPIO_PIN_15; //Wlacz podswietlenie
	if (measDelay == 0) {
		GPIOB->BSRR = GPIO_PIN_6 << 16; //Wylacz LED meas
	}
	GPIOB->BSRR = GPIO_PIN_7 << 16; //Wylacz LED alarm
	RstTimeout();
}

void Tryb2(void) {
	tryb = 2;
	piezo_cnt = 0;
	repeat3 = 0;
	repeat = 0;
	//alarm = 0;
	LCD->FCR &= ~0x00030000;
	temp_odsw |= 0x0002;
	RelOff();
}

void Tryb3(void) {
	tryb = 3;
	RelOff();
	piezo_cnt = 0;
	repeat3 = 0;
	repeat = 0;
	GPIOA->BSRR = GPIO_PIN_15; //Wlacz podswietlenie
	if (measDelay == 0) {
		GPIOB->BSRR = GPIO_PIN_6 << 16; //Wylacz LED meas
	}
	temp_odsw |= 0x0002;
	LCD->FCR |= 0x00030000;
	RstTimeout();
}

void Tryb4(void) {
	tryb = 4;
	repeat = 0;
	repeat3 = 1;
	Alarm();
}

void Tryb5(void) {
	tryb = 5;
	repeat = 0;
	bat_meas = 3;
	GPIOB->BSRR = GPIO_PIN_6 << 16;
	GPIOB->BSRR = GPIO_PIN_7 << 16;
	GPIOA->BSRR = GPIO_PIN_15;
	temp_odsw |= 0x0002;
	HAL_LPTIM_Counter_Stop_IT(&hlptim1);
	if (alarm == 6) {
		Alarm();
	} else {
		PiezoStart(3);
	}
	DispLCD(202);
	LCD->FCR |= 0x00030000;
}

void BtnExti(void) {
	GPIO_InitTypeDef GPIO_InitStruct;
	GPIO_InitStruct.Pin = GPIO_PIN_2;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
	GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_4|GPIO_PIN_5;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
	GPIO_InitStruct.Pin = GPIO_PIN_14;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);
	HAL_NVIC_EnableIRQ(EXTI0_1_IRQn);
  HAL_NVIC_EnableIRQ(EXTI2_3_IRQn);
	HAL_NVIC_EnableIRQ(EXTI4_15_IRQn);
}

void BtnGpio(void) {
	GPIO_InitTypeDef GPIO_InitStruct;
	HAL_NVIC_DisableIRQ(EXTI0_1_IRQn);
	HAL_NVIC_DisableIRQ(EXTI2_3_IRQn);
	HAL_NVIC_DisableIRQ(EXTI4_15_IRQn);
	GPIO_InitStruct.Pin = GPIO_PIN_2;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
	GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_4|GPIO_PIN_5;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
	GPIO_InitStruct.Pin = GPIO_PIN_14;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);
}

void BoostOn(void) {
	if ((GPIOC->IDR & GPIO_PIN_15) == 0) {
		GPIOC->BSRR = GPIO_PIN_15;
		WaitUs(5000);
	}
}

void BoostOff(void) {
	GPIOC->BSRR = GPIO_PIN_13 << 16; //Wylacz piezo
	GPIOA->BSRR = GPIO_PIN_15 << 16; //Wylacz podswietlenie
	GPIOB->BSRR = GPIO_PIN_6 << 16; //Wylacz LED meas
	GPIOB->BSRR = GPIO_PIN_7 << 16; //Wylacz LED alarm
	while (relDelay > 0);
	while (piezoDelay > 0);
	GPIOA->BSRR = GPIO_PIN_11 << 16; //Wylacz zasilanie przekaznika
	GPIOA->BSRR = GPIO_PIN_12 << 16; //Wylacz zasilanie przekaznika
	GPIOC->BSRR = GPIO_PIN_15 << 16; //Wylacz przetwornice BOOST
}

void CtrlTemp(void) {
	//LED i podswietlenie
	if (tryb > 0) {
		if (alarm == 0) {
			if (led_meas) {
				GPIOB->BSRR = GPIO_PIN_6;
			} else {
				GPIOB->BSRR = GPIO_PIN_6 << 16;
			}
			GPIOB->BSRR = GPIO_PIN_7 << 16;
		} else if (alarm == 1) {
			if (led_meas) {
				GPIOB->BSRR = GPIO_PIN_6;
			} else {
				GPIOB->BSRR = GPIO_PIN_6 << 16;
			}
			if (blink) {
				GPIOB->BSRR = GPIO_PIN_7;
				GPIOA->BSRR = GPIO_PIN_15;
			} else {
				GPIOB->BSRR = GPIO_PIN_7 << 16;
				GPIOA->BSRR = GPIO_PIN_15 << 16;
			}
		} else if (alarm == 2) {
			if (blink) {
				GPIOB->BSRR = GPIO_PIN_6;
				GPIOA->BSRR = GPIO_PIN_15;
			} else {
				GPIOB->BSRR = GPIO_PIN_6 << 16;
				GPIOA->BSRR = GPIO_PIN_15 << 16;
			}
			GPIOB->BSRR = GPIO_PIN_7 << 16;
		} else if (alarm == 3) {
			if (led_meas) {
				GPIOB->BSRR = GPIO_PIN_6;
			} else {
				GPIOB->BSRR = GPIO_PIN_6 << 16;
			}
			if (blink) {
				GPIOB->BSRR = GPIO_PIN_7;
			} else {
				GPIOB->BSRR = GPIO_PIN_7 << 16;
			}
		} else if (alarm == 4) {
			if (blink) {
				GPIOB->BSRR = GPIO_PIN_6;
			} else {
				GPIOB->BSRR = GPIO_PIN_6 << 16;
			}
			GPIOB->BSRR = GPIO_PIN_7 << 16;
		} else if (alarm == 6) {
			if (blink) {
				GPIOA->BSRR = GPIO_PIN_15;
			} else {
				GPIOA->BSRR = GPIO_PIN_15 << 16;
			}
		} else if (alarm == 7) {
			GPIOA->BSRR = GPIO_PIN_15 << 16;
			GPIOB->BSRR = GPIO_PIN_6 << 16;
			GPIOB->BSRR = GPIO_PIN_7 << 16;
		}
	}
	//Powtarzanie alarmu
	if (tryb == 4) {
		if ((repeat == 1) & ((alarm == 2) | (alarm == 4))) {
			repeat++;
			RstTimeout();
			Alarm();
		} else if (repeat == 3) {
			if (alarm == 2) {
				tryb_next = 8;
			}
		}
	}
	if ((repeat2 == 1) & ((alarm == 1) | (alarm == 3))) {
		repeat2++;
		if (repeat3) {
			alarm = 1;
			if (tryb == 4) {
				Alarm();
				repeat = 0;
			} else {
				tryb_next = 4;
			}
		} else {
			RstTimeout();
			piezo_cnt = 0;
			PiezoStart(3);
		}
	}
	//Nowy pomiar temperatury
	if (temp_odsw & 0x0001) { 
		temp_odsw &= ~0x0001;
		if (tryb == 0) {
			if (temperature == 201) {
				alarm = 4;
			} else if (temperature >= prog_temp) {
				alarm = 3;
				if (temperature >= (prog_temp + 5)) {
					repeat2 = 2;
				} else {
					repeat2 = 0;
				}
			}
		} else if ((tryb == 1) | (tryb == 3)) {
			if (tryb == 1) {
				DispLCD(temperature);
			}
			if (temperature == 201) {
				alarm = 4;
				PiezoStart(3);
				RstTimeout();
			} else {
				if ((alarm == 4) | (alarm == 2)) {
					alarm = 0;
					piezo_cnt = 0;
					RstTimeout();
				}
				if (alarm == 0) {
					if (temperature >= prog_temp) {
						alarm = 3;
						if (temperature >= (prog_temp + 5)) {
							repeat2 = 2;
						} else {
							repeat2 = 0;
						}
						if (tryb == 1) {
							PiezoStart(3);
						}
						RstTimeout();
					}
				} else if ((alarm == 1) | (alarm == 3)) {
					if (temperature <= (prog_temp - 3)) {
						alarm = 0;
						piezo_cnt = 0;
						RstTimeout();
					} else if (temperature >= (prog_temp + 5)) {
						if (repeat2 == 0) {
							repeat2 = 1;
						}						
					}
				}
			}
		} else if (tryb == 2) {
			DispLCD(temperature);
			if (temperature == 201) {
				if ((alarm == 0) | (alarm == 1) | (alarm == 3)) {
					alarm = 2;
					tryb_next = 4;
				} else {
					tryb_next = 7;
				}
			} else {
				tryb_next = 7;
				if (alarm == 0) {
					if (temperature >= prog_temp) {
						alarm = 1;
						if (temperature >= (prog_temp + 5)) {
							repeat2 = 2;
						} else {
							repeat2 = 0;
						}
						tryb_next = 4;
					}
				} else	if ((alarm == 2) | (alarm == 4)) {
					alarm = 0;
				} else if ((alarm == 1) | (alarm == 3)) {
					if (temperature <= (prog_temp - 3)) {
						alarm = 0;
						piezo_cnt = 0;
					} else if (temperature >= (prog_temp + 5)) {
						if (repeat2 == 0) {
							repeat2 = 1;
							tryb_next = 4;
						}
					}
				}
			}
		} else if (tryb == 4) {
			DispLCD(temperature);
			if (temperature < 201) {
				if ((alarm == 2) | (alarm == 4)) {
					alarm = 0;
					piezo_cnt = 0;
					tryb_next = 1;
					temp_odsw |= 0x0001;
				} else if ((alarm == 1) | (alarm == 3)) {
					if (temperature <= (prog_temp - 3)) {
						alarm = 0;
						piezo_cnt = 0;
						tryb_next = 1;
					} else if (temperature >= (prog_temp + 5)) {
						if (repeat2 == 0) {
							repeat2 = 1;
						}
					}
				}
			} else if ((alarm == 1) | (alarm == 3)) {
				alarm = 2;
				repeat = 0;
				Alarm();
			}
		}
	}
	//Nowa nastawa progu alarmu lub odswiezenie wyswietlacza
	if (temp_odsw & 0x0002) { 
		temp_odsw &= ~0x0002;
		if (tryb == 0) {
			DispLCD(200); //Wszystkie segmenty
		}	else if (tryb == 1) {
			DispLCD(temperature);
		} else if (tryb == 3) {
			DispLCD(prog_temp);
			//Sprawdz nastawe z biezaca temperatura i nie piszcz!
			if (temperature < 201) {
				temp_odsw |= 0x0001;
				alarm = 0;
				blinkDelay = 1;
			}
		} else if (tryb == 5) {
			DispLCD(202);
		}
	}
	//Wylaczanie przekaznika
	if (rel_turnoff) {
		rel_turnoff = 0;
		RelOff();
	}
	//Wychodzenie z alarmu slabej baterii przy podlaczeniu zasilacza
	if (tryb == 5) {
		if (GPIOC->IDR & GPIO_PIN_14) {
			HAL_LPTIM_Counter_Start_IT(&hlptim1, LPT_PERIOD);
			alarm = 0;
			tryb_next = 1;
			bat_meas = 2;
			piezo_cnt = 0;
		}
	}
}

void WaitUs(uint16_t time) {
	TIM21->CNT = 0;
	while(TIM21->CNT<=time);
}

uint16_t OneWireRST(void) {
	uint16_t PRESENCE=0;
	//PB9
	GPIOB->BSRR = GPIO_PIN_9 << 16;
	WaitUs(480);
	GPIOB->BSRR = GPIO_PIN_9;
	WaitUs(70);
	if ((GPIOB->IDR & GPIO_PIN_9)==0) {
		PRESENCE++;
	}
	WaitUs(410);
	if (GPIOB->IDR & GPIO_PIN_9) {
		PRESENCE++;
	}
	if (PRESENCE==2) {
		return 1;
	} else {
		return 0;
	}
}

void OneWireSendBit(uint16_t bit) {
	if (bit) {
		GPIOB->BSRR = GPIO_PIN_9 << 16;
		WaitUs(10);
		GPIOB->BSRR = GPIO_PIN_9;
		WaitUs(65);
	} else {
		GPIOB->BSRR = GPIO_PIN_9 << 16;
		WaitUs(65);
		GPIOB->BSRR = GPIO_PIN_9;
		WaitUs(10);
	}
}

uint16_t OneWireReadBit(void) {
	uint16_t bit;
	GPIOB->BSRR = GPIO_PIN_9 << 16;
	WaitUs(5);
	GPIOB->BSRR = GPIO_PIN_9;
	WaitUs(5);
	if (GPIOB->IDR & GPIO_PIN_9) {
		bit = 1;
	} else {
		bit = 0;
	}
	WaitUs(50);
	return bit;
}

void OneWireSend(uint16_t value) {
	uint16_t pm,i;
	for (i=0;i<8;i++) {
		pm=value >> i;
		pm &= 0x01;
		OneWireSendBit(pm);
	}
}

uint16_t OneWireRead(void) {
	uint16_t i,value=0;
	for (i=0;i<8;i++) {
		if (OneWireReadBit()) {
			value |= 0x01 << i;
		}
	}
	return value;
}

void DispLCD(int16_t value) {
	uint16_t pm = 0;
	uint16_t pm2 = 0;
	uint16_t pm3 = 0;
	if ((value < 200) & (value > (-100))) {
		if (value < 0) {
			pm = -value;
		} else {
			pm = value;
		}
		//cyfra setek
		pm2=(uint16_t)pm/100;
		if (pm2) {
			LCD->RAM[0] = 0x00002200;
		} else {
			LCD->RAM[0] = 0x00000000;
		}
		//cyfra dziesiatek
		pm3=(uint16_t)pm/10;
		pm2=pm2*10;
		pm3=pm3-pm2;
		if (pm3+pm2) {
			LCD->RAM[0] |= Lookup2(pm3);
		}
		//cyfra jednosci
		pm3=pm3+pm2;
		pm3=pm3*10;
		pm3=pm-pm3;
		//Znak minus
		if (value < (-9)) {
			LCD->RAM[0] |= 0x00010000;
		} else if (value < 0) {
			LCD->RAM[0] |= 0x00000100;
		}
		LCD->RAM[0] |= Lookup3(pm3);
	} else if (value == 200) { //wszystkie segmenty
		LCD->RAM[0] = 0x0001FFFF;
	} else if (value == 201) { //brak czujnika - poziome kreski
		LCD->RAM[0] = 0x00011100;
	} else if (value == 202) { //slaba bateria
		LCD->RAM[0] = 0x000014AE;
	} else {
		LCD->RAM[0] = 0x00000000; //(pusty)
	}
	LCD->SR = LCD->SR | LCD_SR_UDR;
}

uint32_t Lookup2(uint16_t cyfra) {
	switch(cyfra) {
		case 0:
			return 0xC4B0;
		case 1:
			return 0x4010;
		case 2:
			return 0xC520;
		case 3:
			return 0xC130;
		case 4:
			return 0x4190;
		case 5:
			return 0x81B0;
		case 6:
			return 0x85B0;
		case 7:
			return 0xC010;
		case 8:
			return 0xC5B0;
		case 9:
			return 0xC1B0;
	}
	return 0;
}

uint32_t Lookup3(uint16_t cyfra) {
	switch(cyfra) {
		case 0:
			return 0x84F;
		case 1:
			return 0x3;
		case 2:
			return 0x104D;
		case 3:
			return 0x1047;
		case 4:
			return 0x1803;
		case 5:
			return 0x1846;
		case 6:
			return 0x184E;
		case 7:
			return 0x43;
		case 8:
			return 0x184F;
		case 9:
			return 0x1847;
	}
	return 0;
}

void RstTimeout(void) {
	switch(tryb) {
		case 0:
			timeout = 33;
			break;
		case 1:
			timeout = 270;
			break;
		case 3:
			timeout = 270;
			break;
		case 4:
			timeout = 6000; //90 sekund - po takim czasie zostanie jednorazowo ponowiony alarm o braku czujnika
			break;
	}
}

void RelOff(void) {
	if (relState) {
		relState = 0;
		relDelay = 3;
		GPIOA->BSRR = GPIO_PIN_11 << 16;
		GPIOA->BSRR = GPIO_PIN_12;
	}
}

void RelOn(void) {
	if (relState == 0) {
		relState = 1;
		relDelay = 3;
		GPIOA->BSRR = GPIO_PIN_12 << 16;
		GPIOA->BSRR = GPIO_PIN_11;
	}
}

void Keys(void) {
	if (GPIOA->IDR & GPIO_PIN_5) { //Nacisnieto przycisk 1
		if (key1 == 0) {
			key1++; //Rozpoczecie eliminacji drgan stykow
		} else if (key1 == 3) {
			//Wykonaj akcje pojedynczego nacisniecia
			Btn1();
			//Rozpoczecie obslugi przytrzymania
			key1++;
		} else if (key1 == 21) {
			//Wykonaj akcje przytrzymania
			Btn1();
			//Kontynuacja petli przytrzymania
			key1++;
		}
	} else { //Przycisk jest zwolniony
		if ((tryb == 3) & (key1 > 0)) {
			LCD->FCR |= 0x00030000;
		}
		key1 = 0;
	}
	
	if (GPIOB->IDR & GPIO_PIN_2) { //Nacisnieto przycisk 2
		if (key2 == 0) {
			key2++;
		} else if (key2 == 3) {
			Btn2();
			key2++;
		} else if (key2 == 21) {
			//Akcja przytrzymania
			Btn2();
			key2++;
		}
	} else {
		if ((tryb == 3) & (key2 > 0)) {
			LCD->FCR |= 0x00030000;
		}
		key2 = 0;
	}
	if (GPIOA->IDR & GPIO_PIN_4) { //Nacisnieto przycisk 3
		if (key3 == 0) {
			key3++;
		} else if (key3 == 3) {
			Btn3();
			key3++;
		} else if (key3 == 21) {
			//Btn3();
			key3++;
		}
	} else {
		key3 = 0;
	}
	if (GPIOA->IDR & GPIO_PIN_0) { //Nacisnieto przycisk 4
		if (key4 == 0) {
			key4++;
		} else if (key4 == 3) {
			Btn4();
			key4++;
		} else if (key4 == 31) {
			if (tryb > 0) {
				tryb_next = 8;
			}
			key4++;
		}
	} else {
		key4 = 0;
	}
}

void Btn1(void) {
	if ((key2 == 0) & (key3 == 0) & (key4 == 0)) {
		RstTimeout();
		repeat3 = 0;
		if (tryb == 3) {
			if (prog_temp > (-50)) {
				prog_temp--;
				temp_odsw |= 0x0002;
				LCD->FCR &= ~0x00030000;
			} else {
				prog_temp = (-50);
			}
		} else if ((tryb == 1) | (tryb == 2)) {
			tryb_next = 3;
		} else if (tryb == 4) {
			if ((alarm == 1) | (alarm == 3)) {
				alarm = 3;
				piezo_cnt = 0;
				tryb_next = 1;
				repeat = 2;
			} else if (alarm == 2) {
				alarm = 4;
				piezo_cnt = 0;
				tryb_next = 1;
			}
		} else if (tryb == 5) {
			tryb_next = 8;
		}
	}
}

void Btn2(void) {
	if ((key1 == 0) & (key3 == 0) & (key4 == 0)) {
		RstTimeout();
		repeat3 = 0;
		if (tryb == 3) {
			if (prog_temp < 120) {
				prog_temp++;
				temp_odsw |= 0x0002;
				LCD->FCR &= ~0x00030000;
			} else {
				prog_temp = 120;
			}
		} else if ((tryb == 1) | (tryb == 2)) {
			tryb_next = 3;
		} else if (tryb == 4) {
			if ((alarm == 1) | (alarm == 3)) {
				alarm = 3;
				piezo_cnt = 0;
				tryb_next = 1;
				repeat = 2;
			} else if (alarm == 2) {
				alarm = 4;
				piezo_cnt = 0;
				tryb_next = 1;
			}
		} else if (tryb == 5) {
			tryb_next = 8;
		}
	}
}

void Btn3(void) {
	if ((key2 == 0) & (key1 == 0) & (key4 == 0)) {
		RstTimeout();
		repeat3 = 0;
		if (tryb == 1) {
			tryb_next = 3;
		} else if ((tryb == 2) | (tryb == 3)) {
			tryb_next = 1;
		} else if (tryb == 4) {
			if ((alarm == 1) | (alarm == 3)) {
				alarm = 3;
				piezo_cnt = 0;
				tryb_next = 1;
				repeat = 2;
			} else if (alarm == 2) {
				alarm = 4;
				piezo_cnt = 0;
				tryb_next = 1;
			}
		} else if (tryb == 5) {
			tryb_next = 8;
		}
	}
}

void Btn4(void) {
	if ((key1 == 0) & (key3 == 0) & (key2 == 0)) {
		RstTimeout();
		repeat3 = 0;
		if (tryb == 4) {
			if ((alarm == 1) | (alarm == 3)) {
				alarm = 3;
				piezo_cnt = 0;
				tryb_next = 1;
				repeat = 2;
			} else if (alarm == 2) {
				alarm = 4;
				piezo_cnt = 0;
				tryb_next = 1;
			}
		} else if (tryb == 5) {
			tryb_next = 8;
		}
	}
}

void ConvertTemp(void) {
	measDelay=9;
	if (OneWireRST()) {
		ds_error = 0;
		OneWireSend(0xCC);
		OneWireSend(0x4E);
		OneWireSend(0x4B);
		OneWireSend(0x46);
		OneWireSend(0x1F);
		if(OneWireRST()) {
			ds_error = 0;
			OneWireSend(0xCC);
			OneWireSend(0x44);
			led_meas = 1;
		} else {
			measDelay = 1;
			ds_error++;
		}
	} else {
		measDelay = 1;
		ds_error++;
		if ((tryb > 0) & (tryb != 4)) {
			led_meas = 0;
		}
	}
}

void Measure(void) {
	uint16_t pm[2];
	if (ds_error < 3) {
		if (measDelay == 1) {
			ConvertTemp();
		} else if (measDelay == 2) {
			measDelay = 0;
			led_meas = 0;
			if (OneWireRST()) {
				ds_error = 0;
				OneWireSend(0xCC);
				OneWireSend(0xBE);
				pm[0] = OneWireRead();
				pm[1] = OneWireRead();
				if ((pm[1] == 0x05) & (pm[0] == 0x50) & (ds_85 < 3)) {
					ds_85++;
					measDelay = 1;
				} else if (OneWireRST()) {
					ds_error = 0;
					ds_85 = 0;
					if (pm[1] & 0xF0) { //Temperatura ujemna
						pm[1] = ~pm[1];
						pm[0] = ~pm[0];
						temperature = pm[1] << 8;
						temperature += pm[0];
						temperature++;
						temperature = temperature / 16;
						temperature = temperature * (-1);
					} else { //Temperatura dodatnia
						temperature = pm[1] << 8;
						temperature += pm[0];
						temperature = temperature / 16;
					}
					temp_odsw |= 0x0001;
					if (temperature != 85) {
						ds_85 = 0;
					}
				} else {
					ds_error++;
					measDelay = 1;
				}
			} else {
				ds_error++;
				measDelay = 1;
			}
		}
	} else {
		ds_error = 0;
		ds_85 = 0;
		measDelay = 1;
		if (temperature < 201) {
			temp_odsw |= 0x0001;
		} else if (tryb == 2) {
			temp_odsw |= 0x0001;
			measDelay = 0;
			led_meas = 0;
		}
		temperature = 201;
	}
}

void StandBy(void) {
	HAL_LPTIM_Counter_Stop_IT(&hlptim1);
	piezo_cnt = 0;
	DispLCD(203);
	RelOff();
	__HAL_PWR_CLEAR_FLAG(PWR_FLAG_WU);
	BoostOff();
	LCD->CR &= ~LCD_CR_LCDEN;
	PWR->CR |= PWR_CR_ULP;
	HAL_ADC_Stop(&hadc);
	while(GPIOA->IDR & GPIO_PIN_0);
	WaitUs(20000);
	HAL_PWR_EnableWakeUpPin(PWR_WAKEUP_PIN1);
	HAL_PWR_EnterSTANDBYMode();
}

void StopMode(void) {
	tryb = 7;
	piezo_cnt = 0;
	BoostOff();
	LPTIM1->CNT = 0;
	HAL_LPTIM_IRQHandler(&hlptim1);
	__HAL_PWR_CLEAR_FLAG(PWR_FLAG_WU);
	if (alarm > 5) {
		LCD->FCR |= 0x00030000;
		DispLCD(202);
	} else {
		LCD->FCR &= ~0x00030000;
	}
	TIM6->CR1 = TIM6->CR1 & ~TIM_CR1_CEN;
	TIM21->CR1 = TIM21->CR1 & ~TIM_CR1_CEN;
	TIM6->CNT = 1;
	TIM6->SR = 0;
	HAL_ADC_Stop(&hadc);
	ADC->CCR &= ~ADC_CCR_VREFEN;
	ADC1->CR &= ~ADC_CR_ADVREGEN;
	DispLCD(temperature);
	BtnExti();
	EXTI->PR = 0x007BFFFF;
	calibrated = 0;
	bat_meas = 0;
	HAL_PWR_EnterSTOPMode(PWR_LOWPOWERREGULATOR_ON, PWR_STOPENTRY_WFI);
	HAL_LPTIM_IRQHandler(&hlptim1);
	HAL_LPTIM_Counter_Start_IT(&hlptim1, LPT_PERIOD);
	BtnGpio();
	TIM6->SR = 0;
	TIM6->CNT = 0;
	TIM6->DIER = TIM6->DIER | TIM_DIER_UIE;
	TIM6->CR1 = TIM6->CR1 | TIM_CR1_CEN;
	TIM21->CR1 = TIM21->CR1 | TIM_CR1_CEN;
	key1 = 100;
	key2 = 100;
	key3 = 100;
	//key4 = 100;
	BoostOn();
}

void PiezoStart(uint16_t beeps) {
	piezo_cnt = 2*beeps;
	piezo_cnt--;
	piezoDelay = 1;
	blinkDelay = 1;
	blink = 0;
}

void Alarm(void) {
	BoostOn();
	RelOn();
	RstTimeout();
	LCD->FCR &= ~0x00030000;
	temp_odsw |= 0x0002;
	GPIOA->BSRR = GPIO_PIN_15; //Wlacz podswietlenie
	if ((alarm == 2) | (alarm == 4)) {
		PiezoStart(60);
	} else if ((alarm == 1) | (alarm == 3)) {
		PiezoStart(60);
	} else if (alarm == 6) {
		PiezoStart(60);
	}
}

void CtrlBat(void) {
	#define VREFINT_CAL ((uint16_t*)((uint32_t) 0x1FF80078))
	uint32_t mV, pm2;
	if (bat_meas == 1) {
		bat_meas = 2;
		if ((GPIOC->IDR & GPIO_PIN_14) == 0) {
			ADC->CCR |= ADC_CCR_VREFEN;
			ADC1->CR |= ADC_CR_ADVREGEN;
			HAL_ADCEx_EnableVREFINT();
			if (calibrated == 0) {
			HAL_ADCEx_Calibration_Start(&hadc, ADC_SINGLE_ENDED);
				calibrated = 1;
			}
			HAL_ADC_Start(&hadc);
			while(__HAL_ADC_GET_FLAG(&hadc, ADC_FLAG_EOC) == 0);
			mV = HAL_ADC_GetValue(&hadc);
			pm2 = 3000*(*VREFINT_CAL);
			mV = pm2/mV;
			
			HAL_ADCEx_DisableVREFINT();
			ADC->CCR &= ~ADC_CCR_VREFEN;
			ADC1->CR &= ~ADC_CR_ADVREGEN;
			if (mV < 2200) {
				bat_meas = 3; //slaba bateria, bedzie alarm!
			}
		} else if (alarm > 5) {
			alarm = 0;
		}
	} else if (GPIOC->IDR & GPIO_PIN_14) {
		bat_meas = 2;
		if (alarm > 5) {
			alarm = 0;
		}
	}
}


/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
  }
  /* USER CODE END Error_Handler */ 
}

#ifdef USE_FULL_ASSERT

/**
   * @brief Reports the name of the source file and the source line number
   * where the assert_param error has occurred.
   * @param file: pointer to the source file name
   * @param line: assert_param error line source number
   * @retval None
   */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
    ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */

}

#endif

/**
  * @}
  */ 

/**
  * @}
*/ 

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
